<?php
namespace xiaofeng;
class MysqlConn
{
    private static $instances = [];
    private $conn = null;
    private static $prefix = "xiaoyewl_";

    private function __construct()
    {
        $config =  require_once __DIR__ . '/../../../config/database.php';

        $dbConfig = $config['connections']['mysql'];

        $this->conn = mysqli_connect(
            "p:" . $dbConfig['hostname'] . ":" . $dbConfig['hostport'],
            $dbConfig['username'],
            $dbConfig['password'],
            $dbConfig['database']
        );

        if (!$this->conn) {
            throw new Exception('数据库连接失败: ' . mysqli_connect_error());
        }

        mysqli_set_charset($this->conn, $dbConfig['charset']);
        MysqlConn::$prefix = $dbConfig['prefix']; 
    }

    public static function getInstance()
    {
        $threadId = getmypid();

        if (!isset(self::$instances[$threadId]) || self::$instances[$threadId] === null) {
            self::$instances[$threadId] = new self();
        }
        return self::$instances[$threadId];
    }
    public static function getPrefix()
    {
        return MysqlConn::$prefix;
    }

    public function query($sql)
    {
        $result = mysqli_query($this->conn, $sql);
        if (!$result) {
            throw new Exception('SQL执行失败: ' . mysqli_error($this->conn));
        }
        return $result;
    }

    public function prepare($sql)
    {
        $stmt = mysqli_prepare($this->conn, $sql);
        if (!$stmt) {
            throw new Exception('SQL预处理失败: ' . mysqli_error($this->conn));
        }
        return $stmt;
    }

    public function execute($stmt)
    {
        if (!$stmt instanceof mysqli_stmt) {
            throw new Exception('无效的语句对象');
        }
        if (!mysqli_stmt_execute($stmt)) {
            return 'SQL执行失败：' . mysqli_error($this->conn);
        }
        return true;
    }

    public function getConnection()
    {
        return $this->conn;
    }

    public function fetch($result)
    {
        return mysqli_fetch_assoc($result);
    }

    public function fetchAll($result)
    {
        $rows = array();
        while ($row = mysqli_fetch_assoc($result)) {
            $rows[] = $row;
        }
        return $rows;
    }

    public function close()
    {
        if ($this->conn) {
            mysqli_close($this->conn);
            $threadId = getmypid();
            if (isset(self::$instances[$threadId])) {
                self::$instances[$threadId] = null;
            }
        }
    }

    private function __clone() {}

    public function __destruct()
    {
        $this->close();
    }
}
